// Project imports:
import 'package:zego_uikit/src/plugins/defines.dart';
import 'package:zego_uikit/zego_uikit.dart';

mixin ZegoUIKitRoomAttributesPluginService {
  Future<ZegoSignalingPluginResult> updateRoomProperty(
    String key,
    String value, {
    bool isForce = false,
    bool isDeleteAfterOwnerLeft = false,
    bool isUpdateOwner = false,
  }) async {
    Map result = await ZegoUIKit()
        .getPlugin(ZegoUIKitPluginType.signaling)!
        .invoke('updateRoomProperty', {
      'key': key,
      'value': value,
      'isForce': isForce,
      'isDeleteAfterOwnerLeft': isDeleteAfterOwnerLeft,
      'isUpdateOwner': isUpdateOwner,
    });

    return ZegoSignalingPluginResult(
      result['errorCode'] as String,
      result['errorMessage'] as String,
      result: result['errorKeys'] as List<String>,
    );
  }

  Future<ZegoSignalingPluginResult> deleteRoomProperties(
    List<String> keys, {
    bool isForce = false,
  }) async {
    Map result = await ZegoUIKit()
        .getPlugin(ZegoUIKitPluginType.signaling)!
        .invoke('deleteRoomProperties', {
      'keys': keys,
      'isForce': isForce,
    });

    return ZegoSignalingPluginResult(
      result["errorCode"] as String,
      result["errorMessage"] as String,
      result: result['errorKeys'] as List<String>,
    );
  }

  Future<ZegoSignalingPluginResult> beginRoomPropertiesBatchOperation({
    bool isForce = false,
    bool isDeleteAfterOwnerLeft = false,
    bool isUpdateOwner = false,
  }) async {
    Map result = await ZegoUIKit()
        .getPlugin(ZegoUIKitPluginType.signaling)!
        .invoke('beginRoomPropertiesBatchOperation', {
      'isForce': isForce,
      'isDeleteAfterOwnerLeft': isDeleteAfterOwnerLeft,
      'isUpdateOwner': isUpdateOwner,
    });

    return ZegoSignalingPluginResult(
      result["errorCode"] as String,
      result["errorMessage"] as String,
    );
  }

  Future<ZegoSignalingPluginResult> endRoomPropertiesBatchOperation() async {
    Map result = await ZegoUIKit()
        .getPlugin(ZegoUIKitPluginType.signaling)!
        .invoke('endRoomPropertiesBatchOperation', {});

    return ZegoSignalingPluginResult(
      result["errorCode"] as String,
      result["errorMessage"] as String,
    );
  }

  Future<ZegoSignalingPluginResult> queryRoomProperties({
    String nextFlag = '',
    int count = 100,
  }) async {
    Map result = await ZegoUIKit()
        .getPlugin(ZegoUIKitPluginType.signaling)!
        .invoke('queryRoomProperties', {});

    return ZegoSignalingPluginResult(
      result["errorCode"] as String,
      result["errorMessage"] as String,
      result: result['roomAttributes'] as Map<String, String>,
    );
  }

  Stream<Map> getRoomPropertiesStream() {
    return ZegoUIKit()
        .getPlugin(ZegoUIKitPluginType.signaling)!
        .getEventStream('roomPropertiesStream');
  }

  Stream<Map> getRoomBatchPropertiesStream() {
    return ZegoUIKit()
        .getPlugin(ZegoUIKitPluginType.signaling)!
        .getEventStream('roomBatchPropertiesStream');
  }
}
